
/*
 * Copyright (C) 2011 Archie L. Cobbs. All rights reserved.
 *
 * $Id: TypedQueryCallback.java 275 2012-02-13 21:56:57Z archie.cobbs $
 */

package org.dellroad.stuff.dao;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

/**
 * Helper class for building and executing JPA typed queries.
 *
 * @param <T> persistent instance type
 * @param <R> query return type
 */
public abstract class TypedQueryCallback<T, R> extends QueryCallback<R> {

    /**
     * Delegates to {@link #executeQuery(TypedQuery)}.
     */
    @SuppressWarnings("unchecked")
    protected final R executeQuery(Query query) {
        return this.executeQuery((TypedQuery<T>)query);
    }

    /**
     * Build the typed query.
     */
    @Override
    protected abstract TypedQuery<T> buildQuery(EntityManager entityManager);

    /**
     * Execute the query.
     */
    protected abstract R executeQuery(TypedQuery<T> query);
}

