
/*
 * Copyright (C) 2012 Archie L. Cobbs. All rights reserved.
 *
 * $Id: PersistentObjectEvent.java 230 2012-01-18 17:08:03Z archie.cobbs $
 */

package org.dellroad.stuff.pobj;

import java.util.EventObject;

/**
 * Notification event emitted by a {@link PersistentObject} to listeners whenever there is an update to the root object.
 *
 * @param <T> type of the root persistent object
 */
@SuppressWarnings("serial")
public class PersistentObjectEvent<T> extends EventObject {

    private final long version;
    private final T oldRoot;
    private final T newRoot;

    public PersistentObjectEvent(PersistentObject<T> persistentObject, long version, T oldRoot, T newRoot) {
        super(persistentObject);
        this.version = version;
        this.oldRoot = oldRoot;
        this.newRoot = newRoot;
    }

    /**
     * Get the {@link PersistentObject} that originated this event.
     */
    @SuppressWarnings("unchecked")
    public PersistentObject<T> getSource() {
        return (PersistentObject<T>)super.getSource();
    }

    /**
     * Get the version that this event is associated with. This will be the version of the {@linkplain #getNewRoot new root}.
     *
     * <p>
     * The {@link PersistentObject} class always delivers notifications in order, so this
     * number should always increase over time.
     */
    public long getVersion() {
        return this.version;
    }

    /**
     * Get the old root prior to the update.
     *
     * <p>
     * The caller must not modify the returned object, as it is shared among all listeners.
     */
    public T getOldRoot() {
        return this.oldRoot;
    }

    /**
     * Get the new root after to the update.
     *
     * <p>
     * The caller must not modify the returned object, as it is shared among all listeners.
     */
    public T getNewRoot() {
        return this.newRoot;
    }
}

